@@ -16,6 +16,7 @@
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 import faulthandler
+from typing import Any
 
 faulthandler.enable()
 from sys import argv
@@ -36,6 +37,11 @@ from bakery import (
     validate_username,
     validate_fullname,
     validate_hostname,
+    check_efi,
+    check_partition_table,
+    list_drives,
+    get_partitions,
+    gen_new_partitions,
     lp,
     setup_translations,
     debounce,
@@ -46,13 +52,13 @@ from bakery import (
     detect_install_device,
     detect_install_source,
     detect_session_configuration,
-    run_deferred,
     reboot,
     upload_log,
     log_path,
+    time_fn,
 )
 from time import sleep
-from datetime import date, datetime, time
+from datetime import datetime
 from babel import dates, numbers
 from babel import Locale as bLocale
 from pyrunning import (
@@ -68,7 +74,7 @@ import config
 
 gi.require_version("Gtk", "4.0")
 gi.require_version("Adw", "1")
-from gi.repository import Gtk, Adw, Gio, GLib
+from gi.repository import Gtk, Adw, Gio, GLib  # type: ignore
 
 # py file path
 script_dir = path.dirname(path.realpath(__file__))
@@ -78,6 +84,7 @@ class BakeryApp(Adw.Application):
     def __init__(self, **kwargs) -> None:
         super().__init__(**kwargs)
         self.connect("activate", self.on_activate)
+        self.css_provider = self.load_css(script_dir + "/data/main.css")
 
     def on_activate(self, app) -> None:
         self.create_action("about", self.on_about_action)
@@ -93,6 +100,7 @@ class BakeryApp(Adw.Application):
         if not win:
             win = BakeryWindow(application=self)
             self.win = win
+            self.add_custom_styling(self.win)
         win.present()
 
     def on_preferences_action(self, widget, _) -> None:
@@ -109,15 +117,18 @@ class BakeryApp(Adw.Application):
             developer_name="BredOS",
             debug_info=self.win.collect_data(show_pass=False),
             version=config.installer_version,
-            developers=["Panda", "bill88t"],
-            designers=["Panda", "DustyDaimler"],
+            developers=["Panda <panda@bredos.org>", "bill88t <bill88t@bredos.org>"],
+            designers=["Panda <panda@bredos.org>", "DustyDaimler"],
+            documenters=["Panda <panda@bredos.org>", "DroidMaster"],
             translator_credits=_("translator-credits"),
             copyright=_("Copyright The BredOS developers"),
             comments=_("Bakery is a simple installer for BredOS"),
             license_type=Gtk.License.GPL_3_0,
             website="https://BredOS.org",
+            issue_url="https://github.com/BredOS/Bakery/issues",
+            support_url="https://discord.gg/jwhxuyKXaa",
         )
-        translators = ["Bill88t", "Panda"]
+        translators = ["Bill88t", "Panda <panda@bredos.org>"]
         about.add_credit_section(_("Translated by"), translators)
         about.add_acknowledgement_section(_("Special thanks to"), ["Shivanandvp"])
         about.present()
@@ -136,6 +147,31 @@ class BakeryApp(Adw.Application):
         if shortcuts:
             self.set_accels_for_action(f"app.{name}", shortcuts)
 
+    def load_css(self, css_fn):
+        """create a provider for custom styling"""
+        global css_provider
+        css_provider = None
+        if css_fn and path.exists(css_fn):
+            css_provider = Gtk.CssProvider()
+            try:
+                css_provider.load_from_path(css_fn)
+            except GLib.Error as e:
+                lp(f"Error loading CSS : {e} ", mode="error")
+                return None
+            lp(f"loading custom styling : {css_fn}", mode="debug")
+        return css_provider
+
+    def _add_widget_styling(self, widget):
+        if css_provider:
+            context = widget.get_style_context()
+            context.add_provider(css_provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)
+
+    def add_custom_styling(self, widget):
+        self._add_widget_styling(widget)
+        # iterate children recursive
+        for child in widget:
+            self.add_custom_styling(child)
+
 
 @Gtk.Template.from_file(script_dir + "/data/window.ui")
 class BakeryWindow(Adw.ApplicationWindow):
@@ -180,7 +216,7 @@ class BakeryWindow(Adw.ApplicationWindow):
         self.cancel_btn.connect("clicked", self.on_cancel_clicked)
         self.err_dialog.connect("response", self.on_err_dialog_response)
         self.log_dialog.connect("response", self.on_log_dialog_response)
-        
+
     def on_err_dialog_response(self, dialog, resp) -> None:
         if resp == "yes":
             self.err_dialog.hide()
@@ -220,6 +256,7 @@ class BakeryWindow(Adw.ApplicationWindow):
             self.log_dialog.hide()
             exit(1)
 
+    @time_fn
     @debounce(2)
     def main_button_clicked(self, button) -> None:
         if button == self.offline_install:
@@ -255,8 +292,7 @@ class BakeryWindow(Adw.ApplicationWindow):
     def on_done_clicked(self, button) -> None:
         # quit the app
         self.close()
-        reboot()  # We are on a hurry passed here!
-        run_deferred()
+        reboot()
 
     @debounce(0.3)
     def on_next_clicked(self, button) -> None:
@@ -293,29 +329,65 @@ class BakeryWindow(Adw.ApplicationWindow):
         num_pages = len(self.pages)
         # if page is user page make it so user cant go forward
         global user_event
+        global part_event
         user_event = threading.Event()
+        part_event = threading.Event()
         self.check_thread = CheckThread(all_pages["User"])
-        if self.current_page == self.pages.index("User"):
-            self.next_btn.set_sensitive(False)
-            # start the thread to check when all fields are filled
-            self.next_btn.set_label(_("Next"))
-            self.check_thread.start()
-        elif self.current_page == self.pages.index("Summary"):
-            user_event.set()
-            all_pages["Summary"].page_shown()
-            # change the next button to install
-            self.next_btn.set_label(_("Install"))
-            self.next_btn.set_sensitive(self.current_page < num_pages - 1)
-            self.back_btn.set_sensitive(self.current_page > 0)
-        elif self.current_page == self.pages.index("Install"):
-            self.back_btn.set_sensitive(False)
-            self.next_btn.set_sensitive(False)
-            self.cancel_btn.set_sensitive(False)
+
+        if self.install_source == "from_iso":
+            self.check_part_thread = CheckPartitioningThread(all_pages["Partitioning"])
+
+            if self.current_page == self.pages.index("User"):
+                self.next_btn.set_sensitive(False)
+                # start the thread to check when all fields are filled
+                self.next_btn.set_label(_("Next"))
+                self.check_thread.start()
+            elif self.current_page == self.pages.index("Partitioning"):
+                user_event.set()
+                self.next_btn.set_sensitive(False)
+                # start the thread to check when all fields are filled
+                self.next_btn.set_label(_("Next"))
+                self.check_part_thread.start()
+            elif self.current_page == self.pages.index("Summary"):
+                user_event.set()
+                all_pages["Summary"].page_shown()
+                # change the next button to install
+                self.next_btn.set_label(_("Install"))
+                self.next_btn.set_sensitive(self.current_page < num_pages - 1)
+                self.back_btn.set_sensitive(self.current_page > 0)
+            elif self.current_page == self.pages.index("Install"):
+                self.back_btn.set_sensitive(False)
+                self.next_btn.set_sensitive(False)
+                self.cancel_btn.set_sensitive(False)
+            else:
+                self.next_btn.set_label(_("Next"))
+                user_event.set()
+                part_event.set()
+                self.next_btn.set_sensitive(self.current_page < num_pages - 1)
+                self.back_btn.set_sensitive(self.current_page > 0)
         else:
-            self.next_btn.set_label(_("Next"))
-            user_event.set()
-            self.next_btn.set_sensitive(self.current_page < num_pages - 1)
-            self.back_btn.set_sensitive(self.current_page > 0)
+            if self.current_page == self.pages.index("User"):
+                self.next_btn.set_sensitive(False)
+                # start the thread to check when all fields are filled
+                self.next_btn.set_label(_("Next"))
+                self.check_thread.start()
+            elif self.current_page == self.pages.index("Summary"):
+                user_event.set()
+                all_pages["Summary"].page_shown()
+                # change the next button to install
+                self.next_btn.set_label(_("Install"))
+                self.next_btn.set_sensitive(self.current_page < num_pages - 1)
+                self.back_btn.set_sensitive(self.current_page > 0)
+            elif self.current_page == self.pages.index("Install"):
+                self.back_btn.set_sensitive(False)
+                self.next_btn.set_sensitive(False)
+                self.cancel_btn.set_sensitive(False)
+            else:
+                self.next_btn.set_label(_("Next"))
+                user_event.set()
+                part_event.set()
+                self.next_btn.set_sensitive(self.current_page < num_pages - 1)
+                self.back_btn.set_sensitive(self.current_page > 0)
 
     def on_cancel_clicked(self, button) -> None:
         # connect the yes button to the delete_pages function
@@ -346,7 +418,7 @@ class BakeryWindow(Adw.ApplicationWindow):
     def get_page_id(self, page_name) -> str:
         return page_name
 
-    def collect_data(self, show_pass=False , *_) -> dict:
+    def collect_data(self, show_pass=False, *_) -> dict:
         data = {}
         if not self.install_type == None:
             install_data = {}
@@ -368,8 +440,14 @@ class BakeryWindow(Adw.ApplicationWindow):
             installer["ui"] = "gui"
             installer["shown_pages"] = self.pages
             data["installer"] = installer
-            data["packages"] = []
-            data["de_packages"] = []
+            data["packages"] = {}
+            if self.install_source == "from_iso":
+                data["packages"]["to_remove"] = config.iso_packages_to_remove
+                data["packages"]["de_packages"] = []
+            try:
+                data["partitions"] = all_pages["Partitioning"].collect_data()
+            except:
+                data["partitions"] = []
             return data
         else:
             return {"install_type": "None"}
@@ -409,10 +487,8 @@ class BakeryWindow(Adw.ApplicationWindow):
             logging_functions=[all_pages["Install"].console_logging],
         )
         self.install_type = install_type
-        lp("Install type: " + self.install_type)
-        lp("Installing on: " + self.install_device)
-        lp("Installation source: " + self.install_source)
         self.current_page = 0
+
         self.update_buttons()
         self.main_stk.set_visible_child(self.install_page.get_child())
 
@@ -636,7 +712,7 @@ class locale_screen(Adw.Bin):
         except Exception as e:
             import traceback
 
-            print(traceback.format_exc())
+            lp(traceback.format_exc(), mode="error")
 
         self.locale_dialog.hide()
 
@@ -675,8 +751,11 @@ class InstallThread(threading.Thread):
 
     def run(self):
         install_data = self.window.collect_data(show_pass=True)
-        lp("Starting install with data: " + str(self.window.collect_data(show_pass=False)))
-        res = bakery.install(install_data, do_deferred=False)
+        lp(
+            "Starting install with data: "
+            + str(self.window.collect_data(show_pass=False))
+        )
+        res = bakery.install(install_data)
         if res == 0:
             # Change to finish page
             self.window.current_page = self.window.pages.index("Finish")
@@ -953,6 +1032,515 @@ class summary_screen(Adw.Bin):
         self.nopasswd.set_active(self.data["user"]["sudo_nopasswd"])
 
 
+class PartitionRow(Adw.ActionRow):
+    def __init__(
+        self,
+        part: dict,
+        part_name: str,
+        size_str: str,
+        size: int,
+        selection: dict,
+        available_parts_box,
+        **kwargs,
+    ):
+        super().__init__(**kwargs)
+        self.partition_info = part
+        self.part_name = part_name
+        self.size = size_str
+        self.available_parts_box = available_parts_box
+        self.selection = selection
+        self.set_title(part_name)
+        self.set_subtitle(size_str)
+        self.set_icon_name("drive-harddisk")
+
+        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
+
+        fs_dropdown = Gtk.DropDown()
+        fs_dropdown.set_valign(Gtk.Align.CENTER)
+        fs_model = Gtk.StringList()
+        fs_dropdown.set_model(fs_model)
+
+        mp_dropdown = Gtk.DropDown()
+        mp_dropdown.set_valign(Gtk.Align.CENTER)
+        mp_model = Gtk.StringList()
+        mp_dropdown.set_model(mp_model)
+
+        mp_model.append("Do not use")
+        fs_model.append("Don't format")
+
+        if size < 50:
+            # Can't use this partition due to size
+            sleep(0.01)
+        elif size < 4000:
+            mp_model.append("Use as boot")
+            fs_model.append("fat32")
+        else:
+            mp_model.append("Use as root")
+            mp_model.append("Use as home")
+            fs_model.append("ext4")
+            fs_model.append("btrfs")
+
+        box.append(fs_dropdown)
+        box.append(mp_dropdown)
+        self.add_suffix(box)
+
+        mp_dropdown.connect("notify::selected-item", self.on_mp_changed)
+        fs_dropdown.connect("notify::selected-item", self.on_fs_changed)
+
+    def change_selection(self, key: Any, value: Any):
+        if not self.part_name in self.selection:
+            self.selection[self.part_name] = {"fs": None, "mp": None}
+        self.selection[self.part_name][key] = value
+
+    def on_fs_changed(self, dropdown, *_):
+        selected = dropdown.props.selected_item
+        if selected is not None:
+            self.fs = selected.props.string
+            self.change_selection("fs", self.fs)
+
+    def on_mp_changed(self, dropdown, *_):
+        selected = dropdown.props.selected_item
+        if selected is not None:
+            mountpoint = selected.props.string
+
+            # Check if another partition is already using this mountpoint
+            for part, sel in self.selection.items():
+                if (
+                    part != self.part_name
+                    and sel["mp"] == mountpoint
+                    and mountpoint != "Do not use"
+                ):
+                    # If mountpoint is already used, show an error or reset dropdown
+                    self.available_parts_box.set_title(
+                        f"Error: Mountpoint '{mountpoint}' is already used by '{part}'"
+                    )
+                    dropdown.set_selected(0)  # Reset to "Do not use"
+                    return
+
+            # Otherwise, apply the mountpoint selection
+            self.mp = mountpoint
+            self.change_selection("mp", self.mp)
+
+
+@time_fn
+@Gtk.Template.from_file(script_dir + "/data/partitioning_screen.ui")
+class partitioning_screen(Adw.Bin):
+    __gtype_name__ = "partitioning_screen"
+
+    sys_type: Gtk.Label = Gtk.Template.Child()
+    part_table: Gtk.Label = Gtk.Template.Child()
+    disk_list: Gtk.DropDown = Gtk.Template.Child()
+    big_grid: Gtk.Grid = Gtk.Template.Child()
+    after_label: Gtk.Label = Gtk.Template.Child()
+    erase_all: Gtk.CheckButton = Gtk.Template.Child()
+    manual_part: Gtk.CheckButton = Gtk.Template.Child()
+    replace_part: Gtk.CheckButton = Gtk.Template.Child()
+    encrypt: Gtk.CheckButton = Gtk.Template.Child()
+    info_label: Gtk.Label = Gtk.Template.Child()
+    guided_partitioning: Gtk.Button = Gtk.Template.Child()
+    manual_partitioning: Gtk.Button = Gtk.Template.Child()
+    term_button: Gtk.Button = Gtk.Template.Child()
+    gparted_button: Gtk.Button = Gtk.Template.Child()
+    stack: Gtk.Stack = Gtk.Template.Child()
+    parts_group: Adw.PreferencesGroup = Gtk.Template.Child()
+    refresh_parts: Gtk.Button = Gtk.Template.Child()
+
+    def __init__(self, window, **kwargs) -> None:
+        super().__init__(**kwargs)
+        self.window = window
+        self.disk_list.connect("notify::selected-item", self.on_disk_selected)
+        self.erase_all.connect("toggled", self.on_partitioning_type)
+        self.manual_part.connect("toggled", self.on_partitioning_type)
+        self.replace_part.connect("toggled", self.on_partitioning_type)
+        self.selected_partition = None
+        self.selectable = False
+        self.selected_mode = None
+        self.selected_mode_view = "guided"
+        self.available_parts_box = None
+        self.stack.set_visible_child_name("guided")
+        self.guided_partitioning.set_visible(False)
+        self.manual_partitioning.set_visible(True)
+
+        self.manual_partitioning.connect("clicked", self.on_manual_partitioning_clicked)
+        self.guided_partitioning.connect("clicked", self.on_guided_partitioning_clicked)
+
+        self.term_button.connect("clicked", self.on_term_button_clicked)
+        self.gparted_button.connect("clicked", self.on_gparted_button_clicked)
+        self.refresh_parts.connect("clicked", self.refresh_parts_clicked)
+
+        self.selection = {}
+
+        self.is_efi = check_efi()
+        self.device = detect_install_device()
+        if self.is_efi:
+            self.sys_type.set_label(_("System type: ") + "UEFI")
+        else:
+            self.sys_type.set_label(_("System type: ") + "BIOS")
+        self.first_run = True
+        self.new_first_run = True
+
+        self.disk_model = Gtk.StringList()
+        self.disk_list.set_model(self.disk_model)
+        self.disks = list_drives()
+        self.disk = list(self.disks.keys())[0]
+        self.all_partitions = get_partitions()
+        for disk in self.disks:
+            self.disk_model.append(disk + ": " + self.disks[disk])
+        partition_table = check_partition_table(self.disk)
+        self.part_table.set_label(_("Partition table: ") + str(partition_table))
+        self.partitions = {}
+        self.partitions[self.disk] = self.all_partitions[self.disk]
+        self.populate_available_parts(self.partitions)
+        self.populate_disk_preview(self.partitions)
+
+    def validate_selection(self) -> bool:
+        # Need atleast 1 / that is either ext4 or btrfs
+        # If also Need atleast 1 /boot/efi that is either fat32
+        # Else need atleast 1 /boot that is either fat32E
+        root_found = False
+        boot_found = False
+        if self.selected_mode_view == "manual":
+            for part, details in self.selection.items():
+                fs = details["fs"]
+                mp = details["mp"]
+
+                # Check for root (/) partition with ext4 or btrfs
+                if mp == "Use as root" and fs in ("ext4", "btrfs"):
+                    root_found = True
+                if mp == "Use as boot" and fs == "fat32":
+                    boot_found = True
+
+            if root_found and boot_found:
+                return True
+        elif self.selected_mode_view == "guided":
+            if self.selected_mode == "erase_all":
+                return True
+        return False
+
+    def refresh_parts_clicked(self, button) -> None:
+        self.populate_available_parts(self.partitions)
+        self.populate_disk_preview(self.partitions)
+
+    def on_term_button_clicked(self, button) -> None:
+        try:
+            lrun(["gtk-open", "org.gnome.Terminal"], wait=False, force=True)
+        except:
+            lrun(["gtk-launch", "kgx"], wait=False, force=True)
+
+    def on_gparted_button_clicked(self, button) -> None:
+        lrun(["gparted"], wait=False, force=True)
+
+    def on_guided_partitioning_clicked(self, button) -> None:
+        self.selected_mode_view = "guided"
+        self.stack.set_visible_child_name("guided")
+        self.manual_partitioning.set_visible(True)
+        self.guided_partitioning.set_visible(False)
+
+    def on_manual_partitioning_clicked(self, button) -> None:
+        self.selected_mode_view = "manual"
+        self.stack.set_visible_child_name("manual")
+        self.manual_partitioning.set_visible(False)
+        self.guided_partitioning.set_visible(True)
+
+    @time_fn
+    def populate_available_parts(self, partitions) -> None:
+        # For each partition, create a PartitionRow
+        self.selection = {}
+        if self.available_parts_box is not None:
+            self.parts_group.remove(self.available_parts_box)
+            self.available_parts_box = Adw.PreferencesGroup()
+        else:
+            self.available_parts_box = Adw.PreferencesGroup()
+        for idx, partition in enumerate(partitions[list(partitions.keys())[0]]):
+            for name, details in partition.items():
+                size, _, _, fstype = details
+                # get the number of the partition from the dicts keys number
+                size_str = f"{size:.2f} MB" if size < 1024 else f"{size / 1024:.2f} GB"
+                row = PartitionRow(
+                    partition,
+                    name,
+                    size_str,
+                    size,
+                    self.selection,
+                    self.available_parts_box,
+                )
+                self.available_parts_box.add(row)
+        self.parts_group.add(self.available_parts_box)
+
+    @time_fn
+    def populate_disk_preview(self, partitions, new=False) -> None:
+        if not new and self.first_run:
+            self.first_run = False
+            self.parts_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=0)
+            self.legend_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
+            self.big_grid.attach(self.parts_box, 1, 0, 1, 1)
+            self.big_grid.attach(self.legend_box, 1, 1, 1, 1)
+            self.legend_box.set_halign(Gtk.Align.CENTER)
+        elif not new and not self.first_run:
+            self.big_grid.remove(self.parts_box)
+            self.big_grid.remove(self.legend_box)
+            self.legend_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
+            self.parts_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=0)
+            self.big_grid.attach(self.parts_box, 1, 0, 1, 1)
+            self.big_grid.attach(self.legend_box, 1, 1, 1, 1)
+            self.legend_box.set_halign(Gtk.Align.CENTER)
+        if new and self.new_first_run:
+            self.after_label.set_visible(True)
+            self.new_first_run = False
+            self.new_legend_box = Gtk.Box(
+                orientation=Gtk.Orientation.HORIZONTAL, spacing=5
+            )
+            self.new_parts_box = Gtk.Box(
+                orientation=Gtk.Orientation.HORIZONTAL, spacing=0
+            )
+            self.big_grid.attach(self.new_parts_box, 1, 2, 1, 1)
+            self.big_grid.attach(self.new_legend_box, 1, 3, 1, 1)
+            self.new_legend_box.set_halign(Gtk.Align.CENTER)
+        elif new and not self.new_first_run:
+            self.big_grid.remove(self.new_parts_box)
+            self.big_grid.remove(self.new_legend_box)
+            self.new_parts_box = Gtk.Box(
+                orientation=Gtk.Orientation.HORIZONTAL, spacing=0
+            )
+            self.new_legend_box = Gtk.Box(
+                orientation=Gtk.Orientation.HORIZONTAL, spacing=5
+            )
+            self.big_grid.attach(self.new_parts_box, 1, 2, 1, 1)
+            self.big_grid.attach(self.new_legend_box, 1, 3, 1, 1)
+            self.new_legend_box.set_halign(Gtk.Align.CENTER)
+
+        total_size = sum(
+            partition[next(iter(partition))][0]
+            for partition in partitions[list(partitions.keys())[0]]
+        )
+
+        sizes = []
+        for partition in partitions[list(partitions.keys())[0]]:
+            for name, details in partition.items():
+                size, _, _, fstype = details
+                literal_size = int((size / total_size) * 650)
+                sizes.append(literal_size)
+
+        min_size = 20
+        total_allocated = sum(sizes)
+        adjusted_sizes = []
+        adjustment_needed = 650 - total_allocated
+
+        for size in sizes:
+            if size < min_size:
+                adjustment_needed -= min_size - size
+                adjusted_sizes.append(min_size)
+            else:
+                adjusted_sizes.append(size)
+
+        if adjustment_needed != 0:
+            proportion_total = sum(size for size in adjusted_sizes if size >= min_size)
+            for i in range(len(adjusted_sizes)):
+                if adjusted_sizes[i] >= min_size:
+                    adjusted_sizes[i] += int(
+                        adjustment_needed * (adjusted_sizes[i] / proportion_total)
+                    )
+
+        final_total = sum(adjusted_sizes)
+        if final_total != 650:
+            difference = 650 - final_total
+            adjusted_sizes[-1] += difference
+
+        for idx, partition in enumerate(partitions[list(partitions.keys())[0]]):
+            for name, details in partition.items():
+                size, _, _, fstype = details
+                is_first = idx == 0
+                is_last = idx == len(partitions[list(partitions.keys())[0]]) - 1
+                literal_size = adjusted_sizes[idx]
+                partition_box = self.make_part_box(
+                    literal_size,
+                    config.colors[idx % len(config.colors)],
+                    idx,
+                    is_first,
+                    is_last,
+                    selectable=(not new),
+                )
+                leg_box = self.make_legend_box(
+                    name, size, fstype, config.colors[idx % len(config.colors)]
+                )
+
+                if not new:
+                    self.parts_box.append(partition_box)
+                    self.legend_box.append(leg_box)
+                    self.parts_box.set_size_request(650, 35)
+                else:
+                    self.new_parts_box.append(partition_box)
+                    self.new_legend_box.append(leg_box)
+                    self.new_parts_box.set_size_request(650, 35)
+
+    def make_part_box(
+        self, size, color, part_num, first=False, last=False, selectable=True
+    ) -> Gtk.Box:
+        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
+        box.set_size_request(int(size), 35)
+        context = box.get_style_context()
+        context.add_provider(css_provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)
+
+        context.add_class("partition-box")
+        context.add_class(color)
+
+        if first:
+            context.add_class("first-partition")
+        if last:
+            context.add_class("last-partition")
+
+        # Add click gesture for selection
+        if selectable:
+            gesture = Gtk.GestureClick.new()
+            gesture.connect("pressed", self.on_partition_box_clicked, box)
+            box.add_controller(gesture)
+
+        box.part_num = part_num
+
+        return box
+
+    def make_legend_box(self, name, real_size, fstype, color) -> Gtk.Box:
+        # size is in mb if size is less than 1024mb show in mb else in gb
+
+        if real_size < 1024:
+            size_str = f"{real_size:.2f} MB"
+        else:
+            size_str = f"{real_size / 1024:.2f} GB"
+
+        size_label = Gtk.Label(
+            label=f'<span foreground="black">{size_str}</span>'
+        )  # Set color to 'black'
+        size_label.set_xalign(0)
+        size_label.set_use_markup(True)
+        try:
+            name = name.split("/dev/")[1]
+        except:
+            pass
+        label = Gtk.Label(
+            label=f'<span foreground="black">{name}</span>'
+        )  # Set color to 'black'
+        label.set_xalign(0)
+        label.set_use_markup(True)
+
+        fstype_label = Gtk.Label(
+            label=f'<span foreground="black">{fstype if fstype else "Unknown"}</span>'
+        )
+        fstype_label.set_xalign(0)
+        fstype_label.set_use_markup(True)
+
+        color_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
+        color_box.set_size_request(16, 16)
+        color_box.get_style_context().add_class(color)
+        # center the color box
+        color_box.set_halign(Gtk.Align.CENTER)
+        # dont vertically expand the color box
+        color_box.set_valign(Gtk.Align.CENTER)
+        context = color_box.get_style_context()
+        context.add_provider(css_provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)
+        legend_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)
+        legend_box.append(color_box)
+        labels_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=5)
+        labels_box.set_valign(Gtk.Align.CENTER)
+        labels_box.append(label)
+        labels_box.append(size_label)
+        labels_box.append(fstype_label)
+        legend_box.append(labels_box)
+        return legend_box
+
+    def on_disk_selected(self, dropdown, *_) -> None:
+        selected = dropdown.props.selected_item
+        if selected is not None:
+            self.disk = selected.props.string.split(":")[0]
+            self.part_table.set_label(
+                "Partition table: " + str(check_partition_table(self.disk))
+            )
+            self.partitions = {}
+            self.partitions[self.disk] = self.all_partitions[self.disk]
+            self.populate_available_parts(self.partitions)
+            self.populate_disk_preview(self.partitions)
+            if self.selected_mode is not None:
+                self.set_mode(self.selected_mode)
+
+    def set_mode(self, mode):
+        self.selected_mode = mode
+        if self.selected_partition is not None:
+            self.selected_partition.get_style_context().remove_class("selected")
+        if mode == "erase_all":
+            self.selectable = False
+            self.info_label.set_label(
+                "A new 256MB EFI partition will be created and the rest will be used for BredOS"
+            )
+            self.new_partitions = gen_new_partitions(self.partitions, "erase_all")
+            self.populate_disk_preview(self.new_partitions, new=True)
+        elif mode == "manual_part":
+            self.info_label.set_label(
+                "This will allow you to manually partition the disk"
+            )
+        elif mode == "replace_part":
+            self.info_label.set_label(
+                "This will replace the current partitions on the disk"
+            )
+            self.selectable = True
+            self.populate_disk_preview(self.partitions, new=True)
+
+    def on_partitioning_type(self, radio, *_) -> None:
+        if radio is self.erase_all:
+            self.set_mode("erase_all")
+        elif radio is self.manual_part:
+            self.set_mode("manual_part")
+        elif radio is self.replace_part:
+            self.set_mode("replace_part")
+
+    def on_partition_box_clicked(self, gesture, n_press, x, y, box):
+        if self.selected_partition is not None:
+            # Remove the selection style from the previously selected partition
+            self.selected_partition.get_style_context().remove_class("selected")
+
+        if self.selectable:
+            # Set the clicked box as the selected partition
+            self.selected_partition = box
+            # Add a selection style to the clicked box
+            box.get_style_context().add_class("selected")
+            print(self.partitions)
+            self.new_partitions = gen_new_partitions(
+                self.partitions, "replace", self.selected_partition.part_num
+            )
+            print(self.new_partitions)
+            self.populate_disk_preview(self.new_partitions, new=True)
+
+    def collect_data(self) -> dict:
+        data = {}
+        data["type"] = self.selected_mode_view
+        data["efi"] = self.is_efi
+        if self.selected_mode_view == "manual":
+            data["disk"] = self.disk
+            data["partitions"] = self.selection
+        else:
+            data["disk"] = self.disk
+            data["mode"] = self.selected_mode
+            try:
+                data["partitions"] = self.new_partitions
+            except AttributeError:
+                data["partitions"] = None
+        return data
+
+
+class CheckPartitioningThread(threading.Thread):
+    def __init__(self, window):
+        threading.Thread.__init__(self)
+        self.window = window
+
+    def run(self):
+        while True:
+            if part_event.is_set():
+                break
+            if self.window.validate_selection():
+                win.next_btn.set_sensitive(True)
+            else:
+                win.next_btn.set_sensitive(False)
+            sleep(0.5)
+
+
 @Gtk.Template.from_file(script_dir + "/data/install_screen.ui")
 class install_screen(Adw.Bin):
     __gtype_name__ = "install_screen"
@@ -966,6 +1554,7 @@ class install_screen(Adw.Bin):
         super().__init__(**kwargs)
         self.window = window
         self.console_buffer = self.console_text_view.get_buffer()
+        self.console_buffer.connect("changed", self.on_text_buffer_changed)
 
         self.colors = {
             "CRITICAL": "#a40000",
@@ -979,6 +1568,11 @@ class install_screen(Adw.Bin):
             None: "#808080",
         }
 
+    def on_text_buffer_changed(self, buffer):
+        # Scroll to the end of the text buffer
+        adj = self.console.get_vadjustment()
+        adj.set_value(adj.get_upper() - adj.get_page_size())
+
     def console_logging(
         self,
         logging_level: int,
